' Microchip PICBASIC PRO Firmware - Serial Communications Demo

' FileName:        serial_port_demo.bas
' Processor:       PIC18
' Compiler:        PICBASIC PRO

' Software License Agreement
'
' Licensor grants any person obtaining a copy of this software ("You") 
' a worldwide, royalty-free, non-exclusive license, for the duration of 
' the copyright, free of charge, to store and execute the Software in a 
' computer system and to incorporate the Software or any portion of it 
' in computer programs You write.   

' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

' Author               Date        Comment
' Jan Axelson           1/2/08     Original
 
' This application demonstrates detecting and responding to defined commands 
' received over the PIC's asynchronous serial port.

' You can communicate with the PIC using a terminal emulator such as 
' HyperTerminal or a custom application. 

' This application and a Visual Basic application to communicate with the PIC 
' are available from www.Lvr.com


' Specify the oscillator frequency in MHz. (Default = 4.)

define OSC 4

' Enable the port and receiver.

define HSER_RCSTA 90h

' Enable the transmitter and set BRGH = 1.

define HSER_TXSTA 24h

' Set the bit rate.

define HSER_BAUD 1200

' Clear serial-port overflow errors automatically.

define HSER_CLROERR

serial_in      	var	BYTE
received_text  	var BYTE[5]

' Port B bit 1 is an output that drives an LED.

TRISB.1 = 0

' Enable unmasked peripheral interrupts  
  
INTCON = %11000000 

' Enable the serial receive interrupt   

PIE1 = %00100000

on interrupt goto receive_serial_data

' The main program loop.

loop:

	' Perform additional actions here.
 
goto loop

end

'************************************************************************
' Routine:        	receive_serial_data 

' PreCondition:    	Data has arrived at the serial port.

' Input:           	None

' Output:          	None

' Side Effects:    	None

' Overview:       	Serial port interrupt service routine.
'				On receiving data without a framing error,
'				wait to receive a line-feed character (0Ah) or 
'				5 bytes or a timeout.
'				On receiving a LF or 5 bytes, call respond_to_command.   
'                  
' Note:            	A valid command is 3 characters + optional CR (0Dh) + LF (0Ah).
'************************************************************************
disable

receive_serial_data:

	if (RCSTA.2 = 1) then

		' A framing error occurred. 
		' Read RCREG to clear the error but don't use the data.

	hserin [serial_in]

	else

		' No framing error occurred
		' Wait for a line-feed code (0Ah), 5 characters, or a timeout.
		' The timeout is 1 sec. because users might be typing commands
		' in a terminal-emulator program.

		hserin 1000, continue, [STR received_text\5\$a]
	
		gosub respond_to_command

		continue:

	endif

resume
enable

'************************************************************************
' Routine:        	respond_to_command 

' PreCondition:    	Data has arrived at the serial port.

' Input:           	None

' Output:          	The command response.

' Side Effects:    	None

' Overview:       	Check if a defined command has been received.
'				If so, execute the command and send a response.   
'                  
' Note:            	Each command ends in a LF code (0Ah). 
'				Defined commands: 
'				L11: turn on the LED at PORTB.1 and return "1" + LF.
'				L10: turn off the LED at PORTB.1 and return "0" + LF. 
'				Ignore a CR code (0Dh) preceding the LF if present.
'************************************************************************
respond_to_command:

	' Check for defined commands.

	if (received_text[0] = "L") AND (received_text[1] = "1") then

		' The command begins with "L1".

		select case received_text[2]

			case "1"
	
				' Turn on the LED.
					 
				high PORTB.1

				' Send a response ending in a line-feed code.
   			
				hserout ["1",10]

			case "0"

				' Turn off the LED.

				low PORTB.1

		 		' Send a response ending in a line-feed code.
			
   				hserout ["0",10]				

			case else
	
		end select

	endif

return    
